﻿//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Win32;
using DDay.iCal;
using DDay.iCal.Components;
using DDay.iCal.DataTypes;
using DDay.iCal.Serialization;
using DDay.iCal.Serialization.iCalendar.DataTypes;
using Duration = DDay.iCal.DataTypes.Duration;
using Trigger = DDay.iCal.DataTypes.Trigger;


namespace MindFusion.Scheduling.Wpf.Samples.CS.ICalendarFormat
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			calendar.BeginInit();
			calendar.CurrentView = CalendarView.SingleMonth;
			calendar.MonthSettings.DaySettings.HeaderSize = 24;
			calendar.EndInit();
		}

		private void Hyperlink_Click(object sender, RoutedEventArgs e)
		{
			LicenseWindow window = new LicenseWindow();
			window.Owner = this;
			window.ShowDialog();
		}

		private void clear_Click(object sender, RoutedEventArgs e)
		{
			calendar.Schedule.Clear();
		}

		private void save_Click(object sender, RoutedEventArgs e)
		{
			SaveFileDialog dialog = new SaveFileDialog();
			dialog.Filter = "ICalendar files (*.ics)|*.ics|All files|*.*||";
			if (dialog.ShowDialog(this) == true)
				SaveICalendar(dialog.FileName);
		}

		private void load_Click(object sender, RoutedEventArgs e)
		{
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.Filter = "ICalendar files (*.ics)|*.ics|All files|*.*||";
			if (dialog.ShowDialog(this) == true)
				LoadICalendar(dialog.FileName);
		}

		private void calendar_ItemClick(object sender, ItemMouseEventArgs e)
		{
			if (e.Clicks == 2 &&
				e.LeftButton == MouseButtonState.Pressed &&
				e.RightButton == MouseButtonState.Released)
			{
				// Cancel the current drag operation
				calendar.ResetDrag();

				// Edit the clicked item
				EditItem(e.Item as Appointment);
			}
		}

		/// <summary>
		/// Shows the appointment form for the specified item.
		/// </summary>
		private void EditItem(Appointment app)
		{
			// Display the appointment editing form
			AppointmentForm form = new AppointmentForm(calendar.Schedule);
			form.SetAppointment(app);
			form.ShowDialog();
		}

		private DaysOfWeek ToDaysOfWeek(DayOfWeek d)
		{
			switch (d)
			{

				case DayOfWeek.Monday:
					return DaysOfWeek.Monday;

				case DayOfWeek.Tuesday:
					return DaysOfWeek.Tuesday;

				case DayOfWeek.Friday:
					return DaysOfWeek.Friday;

				case DayOfWeek.Saturday:
					return DaysOfWeek.Saturday;

				case DayOfWeek.Sunday:
					return DaysOfWeek.Sunday;

				case DayOfWeek.Wednesday:
					return DaysOfWeek.Wednesday;

				case DayOfWeek.Thursday:
					return DaysOfWeek.Thursday;

			}

			return DaysOfWeek.None;
		}

		private DayOfWeek FromDaysOfWeek(DaysOfWeek d)
		{
			switch (d)
			{

				case DaysOfWeek.Monday:
					return DayOfWeek.Monday;

				case DaysOfWeek.Tuesday:
					return DayOfWeek.Tuesday;

				case DaysOfWeek.Friday:
					return DayOfWeek.Friday;

				case DaysOfWeek.Saturday:
					return DayOfWeek.Saturday;

				case DaysOfWeek.Sunday:
					return DayOfWeek.Sunday;

				case DaysOfWeek.Wednesday:
					return DayOfWeek.Wednesday;

				case DaysOfWeek.Thursday:
					return DayOfWeek.Thursday;

			}

			return DayOfWeek.Monday;
		}

		private DayOfWeekType ToDayOfWeekType(DayOfWeek d)
		{
			switch (d)
			{

				case DayOfWeek.Monday:
					return DayOfWeekType.Monday;

				case DayOfWeek.Tuesday:
					return DayOfWeekType.Tuesday;

				case DayOfWeek.Friday:
					return DayOfWeekType.Friday;

				case DayOfWeek.Saturday:
					return DayOfWeekType.Saturday;

				case DayOfWeek.Sunday:
					return DayOfWeekType.Sunday;

				case DayOfWeek.Wednesday:
					return DayOfWeekType.Wednesday;

				case DayOfWeek.Thursday:
					return DayOfWeekType.Thursday;

			}

			return DayOfWeekType.Monday;
		}

		private DayOfWeek FromDayOfWeekType(DayOfWeekType d)
		{
			switch (d)
			{

				case DayOfWeekType.Monday:
					return DayOfWeek.Monday;

				case DayOfWeekType.Tuesday:
					return DayOfWeek.Tuesday;

				case DayOfWeekType.Friday:
					return DayOfWeek.Friday;

				case DayOfWeekType.Saturday:
					return DayOfWeek.Saturday;

				case DayOfWeekType.Sunday:
					return DayOfWeek.Sunday;

				case DayOfWeekType.Wednesday:
					return DayOfWeek.Wednesday;

				case DayOfWeekType.Thursday:
					return DayOfWeek.Thursday;

			}

			return DayOfWeek.Monday;
		}

		/// <summary>
		/// Obtains a location with the specified name.
		/// </summary>
		private Location GetLocation(string name)
		{
			foreach (Location location in calendar.Schedule.Locations)
			{
				if (location.Name == name)
					return location;
			}

			return null;
		}

		/// <summary>
		/// Obtains a contact with the specified name.
		/// </summary>
		private Contact GetContact(string name)
		{
			foreach (Contact contact in calendar.Schedule.Contacts)
			{
				if (contact.Name == name)
					return contact;
			}

			return null;
		}

		/// <summary>
		/// Registers the specified event as a master item of a recurrence.
		/// </summary>
		private void RegisterRecurrenceMaster(string id, Item item)
		{
			List<Item> list = null;
			if (recurrentItems.ContainsKey(id))
			{
				list = recurrentItems[id] as List<Item>;
			}
			else
			{
				list = new List<Item>();
				recurrentItems[id] = list;
			}

			list.Add(item);
		}

		/// <summary>
		/// Copies all non-recurrent properties from the specified i-event
		/// to the specified MindFusion.Scheduling item.
		/// </summary>
		private void CopyPropertiesNoRec(Item item, Event ievent)
		{
			// Transfer general properties
			item.StartTime = ievent.Start.Value;
			item.EndTime = ievent.End.Value;
			item.AllDayEvent = ievent.IsAllDay;

			if (ievent.Summary != null)
				item.HeaderText = ievent.Summary.Value;

			if (ievent.Description != null)
				item.DescriptionText = ievent.Description.Value;

			if (ievent.Attendee != null)
			{
				foreach (Cal_Address attendee in ievent.Attendee)
				{
					string contactName = attendee.Value;

					// Check if there is already a contact with that name in the schedule
					Contact contact = GetContact(contactName);

					// Check if the contact was found
					if (contact == null)
					{
						// Create a new Contact object
						contact = new Contact();
						contact.Name = contactName;

						// Attempt to parse the contactName
						if (contactName.IndexOf(",") != -1)
						{
							string[] parts = contactName.Split(new char[] { ',' },
								StringSplitOptions.RemoveEmptyEntries);
							if (parts.Length > 1)
							{
								contact.FirstName = parts[1].Trim();
								contact.LastName = parts[0].Trim();
							}
							else if (parts.Length > 0)
							{
								contact.FirstName = parts[0].Trim();
							}
						}
						else
						{
							string[] parts = contactName.Split(new char[] { ' ' },
								StringSplitOptions.RemoveEmptyEntries);
							if (parts.Length > 1)
							{
								contact.FirstName = parts[0].Trim();
								contact.LastName = parts[1].Trim();
							}
							else if (parts.Length > 0)
							{
								contact.FirstName = parts[0].Trim();
							}
						}

						if (attendee.EmailAddress != null)
							contact.Email = attendee.EmailAddress;

						// Add the contact to the schedule
						calendar.Schedule.Contacts.Add(contact);
					}

					// Associate the item with this contact
					item.Contacts.Add(contact);
				}
			}

			// Transfer location
			if (ievent.Location != null)
			{
				string locationName = ievent.Location.Value;

				// Check if there is already a location with that name in the schedule
				Location location = GetLocation(locationName);

				// Check if the location was found
				if (location == null)
				{
					// Create a new Location object
					location = new Location();
					location.Name = locationName;

					// Add the location to the schedule
					calendar.Schedule.Locations.Add(location);
				}

				// Associate the location with the appointment
				item.Location = location;
			}

			// Transfer reminder
			if (ievent.Alarms.Count > 0)
			{
				Alarm ialarm = ievent.Alarms[0];

				Reminder reminder = new Reminder();

				try
				{
					reminder.Message = ialarm.Description;
				}
				catch
				{
					// Alarm.Description throws an exception now and then - consume it
				}

				// Check the type of the reminder
				if (ialarm.Trigger.IsRelative)
				{
					// Check the relation type
					if (ialarm.Trigger.Related == Trigger.TriggerRelation.START)
					{
						// Its relative to the start of the event
						reminder.Type = ReminderType.Leading;
						reminder.TimeInterval = -ialarm.Trigger.Duration.Value;
					}
					else
					{
						// Its relative to the end of the event.
						// However, since MindFusion.Scheduling does not support
						// such kind of reminder relation, we will calculate
						// the same interval, relative to the start of the
						// item
						reminder.Type = ReminderType.Leading;
						reminder.TimeInterval = ialarm.Trigger.Duration.Value +
							(item.EndTime - item.StartTime);
					}
				}
				else
				{
					// Its a fixed reminder
					reminder.Type = ReminderType.Exact;
					reminder.Time = ialarm.Trigger.DateTime.Value;
				}

				// Associate the reminder with the item
				item.Reminder = reminder;
			}
		}

		/// <summary>
		/// Copies all non-recurrent properties from the specified
		/// MindFusion.Scheduling item to the specified i-event.
		/// </summary>
		private void CopyPropertiesNoRec(Event ievent, Item item)
		{
			// Transfer general properties
			ievent.Start = new Date_Time(item.StartTime);
			ievent.End = new Date_Time(item.EndTime);
			ievent.IsAllDay = item.AllDayEvent;

			if (item.HeaderText != "")
				ievent.Summary = new DDay.iCal.DataTypes.Text(item.HeaderText);

			if (item.DescriptionText != "")
				ievent.Description = new DDay.iCal.DataTypes.Text(item.DescriptionText);

			// Transfer contacts
			if (item.Contacts.Count > 0)
			{
				ievent.Attendee = new Cal_Address[item.Contacts.Count];
				for (int i = 0; i < item.Contacts.Count; i++)
				{
					Cal_Address contact = new Cal_Address();

					contact.Value = item.Contacts[i].Name;

					ievent.Attendee[i] = contact;
				}
			}

			// Transfer location
			if (item.Location != null)
			{
				ievent.Location = item.Location.Name;
			}

			// Transfer reminder
			if (item.Reminder != null)
			{
				Alarm alarm = new Alarm(ievent);
				alarm.Trigger = new Trigger();

				if (item.Reminder.Type == ReminderType.Leading)
				{
					alarm.Trigger.Related = Trigger.TriggerRelation.START;
					alarm.Trigger.Duration = new Duration(-item.Reminder.TimeInterval);
					alarm.Trigger.DateTime = null;
				}
				else
				{
					alarm.Trigger.DateTime = new Date_Time(item.Reminder.Time);
					alarm.Trigger.Duration = null;
				}

				ievent.Alarms.Add(alarm);
			}
		}

		/// <summary>
		/// Loads the ICalendar data contained in the specified file.
		/// </summary>
		private void LoadICalendar(string fileName)
		{
			// Clear the current schedule
			calendar.Schedule.Clear();

			// Map between the uid's of recurrent items and
			// the actual MindFusion.Scheduling recurrence masters
			recurrentItems = new Dictionary<string, List<Item>>();

			// A list of recurrence exceptions - processed
			// after all events are intially loaded
			List<Event> exceptions = new List<Event>();

			// A list of deleted recurrence exceptions - the
			// id of the master item and the date of the deleted exception
			Dictionary<string, List<DateTime>> deletedExceptions = new Dictionary<string, List<DateTime>>();

			iCalendar icalendar = iCalendar.LoadFromFile(fileName);
			foreach (UniqueComponent component in icalendar.UniqueComponents)
			{
				if (!(component is Event))
					continue;

				Event ievent = component as Event;

				// Check first if the event is a recurrence exception.
				// The exception might come before the actual master item,
				// so handle this by processing exceptions at the end
				if (ievent.Properties != null && ievent.Properties.Count > 0)
				{
					if (ievent.Properties.ContainsKey("RECURRENCE-ID"))
					{
						// Register for subsequent processing
						exceptions.Add(ievent);

						// Skip further processing of this event for now
						continue;
					}
				}

				Appointment app = new Appointment();

				// Transfer all properties, par recurrence
				CopyPropertiesNoRec(app, ievent);

				// Check for deleted exceptions
				if (ievent.ExDate != null)
				{
					List<DateTime> deletedDates = new List<DateTime>();
					foreach (RDate date in ievent.ExDate)
					{
						if (date.Periods == null || date.Periods.Count == 0)
							continue;

						deletedDates.Add(date.Periods[0].StartTime.Value);
					}

					deletedExceptions[ievent.UID.Value] = deletedDates;
				}

				// Transfer recurrence
				if (ievent.RRule != null)
				{
					int irule = 0;
					foreach (Recur rule in ievent.RRule)
					{
						Recurrence recurrence = new Recurrence();
						recurrence.StartDate = ievent.DTStart.Value;

						// The recurrence end type
						if (rule.Count > 0)
						{
							recurrence.NumOccurrences = rule.Count;
							recurrence.RecurrenceEnd = RecurrenceEnd.NumOccurrences;
						}
						else if (rule.Until != null)
						{
							recurrence.EndDate = rule.Until.Value;
							recurrence.RecurrenceEnd = RecurrenceEnd.EndDate;
						}
						else
						{
							recurrence.RecurrenceEnd = RecurrenceEnd.Never;
						}

						// The recurrence pattern
						if (rule.Frequency == Recur.FrequencyType.DAILY)
						{
							if (rule.ByDay == null || rule.ByDay.Count == 0)
							{
								recurrence.Pattern = RecurrencePattern.Daily;
								recurrence.DailyRecurrence = DailyRecurrence.ByDayInterval;
								recurrence.Days = rule.Interval;
							}
							else
							{
								recurrence.Pattern = RecurrencePattern.Weekly;
								recurrence.Weeks = 1;

								// It would actually be a list of weekly patterns
								// in MindFusion.Scheduling
								for (int i = 0; i < rule.ByDay.Count; i++)
								{
									recurrence.DaysOfWeek = ToDaysOfWeek(rule.ByDay[i].DayOfWeek);

									// The first recurrence is assigned to the appointment being
									// loaded. Subsequent recurrences are assigned to clones of
									// this appointment, since MindFusion.Scheduling cannot support this
									// type of recurrence patterns
									if (i == 0)
									{
										app.Recurrence = recurrence.Clone() as Recurrence;

										// Register the clone
										calendar.Schedule.Items.Add(app);

										// Register this item as a recurrence item under the same id
										RegisterRecurrenceMaster(ievent.UID.Value, app);
									}
									else
									{
										// Create a clone of the current appointment and assign
										// a clone of the recurrence to it
										Appointment clone = app.Clone() as Appointment;
										clone.Recurrence = recurrence.Clone() as Recurrence;

										// Register the clone
										calendar.Schedule.Items.Add(clone);

										// Register this item as a recurrence item under the same id
										RegisterRecurrenceMaster(ievent.UID.Value, clone);
									}
								}

								// Skip further processing
								continue;
							}
						}
						else if (rule.Frequency == Recur.FrequencyType.WEEKLY)
						{
							recurrence.Pattern = RecurrencePattern.Weekly;
							recurrence.Weeks = rule.Interval;

							DaysOfWeek daysOfWeek = DaysOfWeek.None;
							foreach (Recur.DaySpecifier day in rule.ByDay)
							{
								daysOfWeek = ToDaysOfWeek(day.DayOfWeek) | daysOfWeek;
							}
							recurrence.DaysOfWeek = daysOfWeek;
						}
						else if (rule.Frequency == Recur.FrequencyType.MONTHLY)
						{
							recurrence.Pattern = RecurrencePattern.Monthly;
							recurrence.Months = rule.Interval;

							if (rule.ByDay != null && rule.ByDay.Count > 0)
							{
								recurrence.MonthlyRecurrence = MonthlyRecurrence.ByDayType;
								for (int i = 0; i < rule.ByDay.Count; i++)
								{
									recurrence.Day = ToDayOfWeekType(rule.ByDay[i].DayOfWeek);
									switch (rule.ByDay[i].Num)
									{

										case -1: // Last
											recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Last;
											break;

										case 1:
											recurrence.Occurrence = MindFusion.Scheduling.Occurrence.First;
											break;

										case 2:
											recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Second;
											break;

										case 3:
											recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Third;
											break;

										case 4:
											recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Fourth;
											break;

									}

									// The first recurrence is assigned to the appointment being
									// loaded. Subsequent recurrences are assigned to clones of
									// this appointment, since MindFusion.Scheduling cannot support this
									// type of recurrence patterns
									if (i == 0)
									{
										app.Recurrence = recurrence.Clone() as Recurrence;

										// Register the clone
										calendar.Schedule.Items.Add(app);

										// Register this item as a recurrence item under the same id
										RegisterRecurrenceMaster(ievent.UID.Value, app);
									}
									else
									{
										// Create a clone of the current appointment and assign
										// a clone of the recurrence to it
										Appointment clone = app.Clone() as Appointment;
										clone.Recurrence = recurrence.Clone() as Recurrence;

										// Register the clone
										calendar.Schedule.Items.Add(clone);

										// Register this item as a recurrence item under the same id
										RegisterRecurrenceMaster(ievent.UID.Value, clone);
									}
								}

								// Skip further processing
								continue;
							}
							else if (rule.ByMonthDay != null && rule.ByMonthDay.Count > 0)
							{
								recurrence.MonthlyRecurrence = MonthlyRecurrence.ByDayNumber;
								for (int i = 0; i < rule.ByMonthDay.Count; i++)
								{
									recurrence.DayOfMonth = rule.ByMonthDay[i];

									// The first recurrence is assigned to the appointment being
									// loaded. Subsequent recurrences are assigned to clones of
									// this appointment, since MindFusion.Scheduling cannot support this
									// type of recurrence patterns
									if (i == 0)
									{
										app.Recurrence = recurrence.Clone() as Recurrence;

										// Register the clone
										calendar.Schedule.Items.Add(app);

										// Register this item as a recurrence item under the same id
										RegisterRecurrenceMaster(ievent.UID.Value, app);
									}
									else
									{
										// Create a clone of the current appointment and assign
										// a clone of the recurrence to it
										Appointment clone = app.Clone() as Appointment;
										clone.Recurrence = recurrence.Clone() as Recurrence;

										// Register the clone
										calendar.Schedule.Items.Add(clone);

										// Register this item as a recurrence item under the same id
										RegisterRecurrenceMaster(ievent.UID.Value, clone);
									}
								}

								// Skip further processing
								continue;
							}
							else
							{
							}
						}
						else if (rule.Frequency == Recur.FrequencyType.YEARLY)
						{
							recurrence.Pattern = RecurrencePattern.Yearly;
						}

						// The first recurrence is assigned to the appointment being
						// loaded. Subsequent recurrences are assigned to clones of
						// this appointment, since MindFusion.Scheduling cannot support this
						// type of recurrence patterns
						if (irule == 0)
						{
							app.Recurrence = recurrence as Recurrence;

							// Register the item
							calendar.Schedule.Items.Add(app);

							// Register the item if it is a recurrence master
							if (ievent.RRule != null && ievent.RRule.Length > 0)
							{
								if (ievent.UID != null)
									RegisterRecurrenceMaster(ievent.UID.Value, app);
							}
						}
						else
						{
							// Create a clone of the current appointment and assign
							// a clone of the recurrence to it
							Appointment clone = app.Clone() as Appointment;
							clone.Recurrence = recurrence.Clone() as Recurrence;

							// Register the clone
							calendar.Schedule.Items.Add(clone);

							// Register this item as a recurrence item under the same id
							RegisterRecurrenceMaster(ievent.UID.Value, clone);
						}

						irule++;
					}
				}
				else
				{
					// Register the clone
					calendar.Schedule.Items.Add(app);
				}
			}

			// Now that all events are loaded, process recurrence exceptions
			foreach (Event ievent in exceptions)
			{
				Property property = ievent.Properties["RECURRENCE-ID"] as Property;
				Date_Time originalDate = new Date_Time(property.Value);

				// Get the recurrence master(s). It is possible to have
				// more than one master if the recurrence pattern cannot
				// be represented with a single recurrence in MindFusion.Scheduling
				List<Item> masters = recurrentItems[ievent.UID.Value] as List<Item>;
				if (masters == null)
					continue;

				if (masters.Count == 0)
					continue;

				// Attempt to generate occurrences for each recurrence in the list
				foreach (Item master in masters)
				{
					if (master.Recurrence == null)
						continue;

					ItemCollection items = master.Recurrence.GenerateItems(
						originalDate.Value, originalDate.Value);

					if (items.Count == 0)
						continue;

					// Make the exception by setting the occurrence's properties
					CopyPropertiesNoRec(items[0], ievent);

					break;
				}
			}

			// Process deleted exceptions as well
			foreach (string uid in deletedExceptions.Keys)
			{
				// Get the recurrence master(s). It is possible to have
				// more than one master if the recurrence pattern cannot
				// be represented with a single recurrence in MindFusion.Scheduling
				List<Item> masters = recurrentItems[uid] as List<Item>;
				if (masters == null)
					continue;

				if (masters.Count == 0)
					continue;

				// Attempt to generate occurrences for each recurrence in the list
				foreach (DateTime deletedDate in deletedExceptions[uid] as List<DateTime>)
				{
					foreach (Item master in masters)
					{
						if (master.Recurrence == null)
							continue;

						ItemCollection items = master.Recurrence.GenerateItems(
							deletedDate, deletedDate);

						if (items.Count == 0)
							continue;

						// Mark this exception as deleted
						master.Recurrence.MarkException(items[0], true);

						break;
					}
				}
			}
		}

		/// <summary>
		/// Saves the calendar in an ICalendar format in the specified file.
		/// </summary>
		private void SaveICalendar(string fileName)
		{
			iCalendar icalendar = new iCalendar();

			icalendar.ProductID = "MindFusion.Scheduling Control";

			foreach (Item item in calendar.Schedule.Items)
			{
				Event ievent = icalendar.Create<Event>();

				CopyPropertiesNoRec(ievent, item);

				if (item.Recurrence != null)
				{
					Recur rule = new Recur();

					// The recurrence end type
					switch (item.Recurrence.RecurrenceEnd)
					{

						case RecurrenceEnd.EndDate:
							rule.Count = 0;
							rule.Until = new Date_Time(item.Recurrence.EndDate);
							break;

						case RecurrenceEnd.Never:
							rule.Count = 0;
							rule.Until = null;
							break;

						case RecurrenceEnd.NumOccurrences:
							rule.Count = item.Recurrence.NumOccurrences;
							rule.Until = null;
							break;

					}

					switch (item.Recurrence.Pattern)
					{

						case RecurrencePattern.Daily:
							{
								rule.Frequency = Recur.FrequencyType.DAILY;

								switch (item.Recurrence.DailyRecurrence)
								{

									case DailyRecurrence.ByDayInterval:
										{
											rule.Interval = item.Recurrence.Days;
										}
										break;

									case DailyRecurrence.EveryWeekend:
										{
											rule.ByDay = new List<Recur.DaySpecifier>();
											rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Saturday));
											rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Sunday));
										}
										break;

									case DailyRecurrence.EveryWorkday:
										{
											rule.ByDay = new List<Recur.DaySpecifier>();
											rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Monday));
											rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Tuesday));
											rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Wednesday));
											rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Thursday));
											rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Friday));
										}
										break;

								}
							}
							break;

						case RecurrencePattern.Weekly:
							{
								rule.Frequency = Recur.FrequencyType.WEEKLY;
								rule.Interval = item.Recurrence.Weeks;

								if ((item.Recurrence.DaysOfWeek & DaysOfWeek.Monday) > 0)
									rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Monday));

								if ((item.Recurrence.DaysOfWeek & DaysOfWeek.Tuesday) > 0)
									rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Tuesday));

								if ((item.Recurrence.DaysOfWeek & DaysOfWeek.Wednesday) > 0)
									rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Wednesday));

								if ((item.Recurrence.DaysOfWeek & DaysOfWeek.Thursday) > 0)
									rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Thursday));

								if ((item.Recurrence.DaysOfWeek & DaysOfWeek.Friday) > 0)
									rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Friday));

								if ((item.Recurrence.DaysOfWeek & DaysOfWeek.Saturday) > 0)
									rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Saturday));

								if ((item.Recurrence.DaysOfWeek & DaysOfWeek.Sunday) > 0)
									rule.ByDay.Add(new Recur.DaySpecifier(DayOfWeek.Sunday));
							}
							break;

						case RecurrencePattern.Monthly:
							{
								rule.Frequency = Recur.FrequencyType.MONTHLY;
								rule.Interval = item.Recurrence.Months;

								switch (item.Recurrence.MonthlyRecurrence)
								{

									case MonthlyRecurrence.ByDayType:
										{
											Recur.DaySpecifier day = new Recur.DaySpecifier();

											day.DayOfWeek = FromDayOfWeekType(item.Recurrence.Day);

											switch (item.Recurrence.Occurrence)
											{

												case MindFusion.Scheduling.Occurrence.Last:
													day.Num = -1;
													break;

												case MindFusion.Scheduling.Occurrence.First:
													day.Num = 1;
													break;

												case MindFusion.Scheduling.Occurrence.Second:
													day.Num = 2;
													break;

												case MindFusion.Scheduling.Occurrence.Third:
													day.Num = 3;
													break;

												case MindFusion.Scheduling.Occurrence.Fourth:
													day.Num = 4;
													break;

											}

											rule.ByDay.Add(day);
										}
										break;

									case MonthlyRecurrence.ByDayNumber:
										{
											rule.ByMonthDay.Add(item.Recurrence.DayOfMonth);
										}
										break;

								}
							}
							break;

						case RecurrencePattern.Yearly:
							{
								rule.Frequency = Recur.FrequencyType.YEARLY;
							}
							break;

					}

					// Mark deleted exceptions
					List<RecurrenceException> deleted = new List<RecurrenceException>();
					foreach (RecurrenceException exception in item.Recurrence.Exceptions)
					{
						if (exception.IsDeleted)
							deleted.Add(exception);
					}

					if (deleted.Count > 0)
					{
						RDate[] deletedExceptions = new RDate[deleted.Count];
						for (int i = 0; i < deleted.Count; i++)
							ievent.AddSingleException(new Date_Time(deleted[i].Item.StartTime));
					}

					// Process non-deleted exceptions
					foreach (RecurrenceException exception in item.Recurrence.Exceptions)
					{
						if (exception.IsDeleted)
							continue;

						// Create a new item representing the exception...
						Event revent = icalendar.Create<Event>();

						CopyPropertiesNoRec(revent, exception.Item);

						bool isDeleted = false;
						Item original = item.Recurrence.GetOccurrence(
							exception.Item.OccurrenceIndex, ref isDeleted, true);

						Date_TimeSerializer dtserializer =
							new Date_TimeSerializer(new Date_Time(original.StartTime));

						string originalDate = dtserializer.SerializeToString();

						Property property = new Property(revent, "RECURRENCE-ID", originalDate);

						// Use the same UID as the master of the recurrence
						revent.UID = new DDay.iCal.DataTypes.Text(ievent.UID.Value);
					}

					ievent.AddRecurrence(rule);
				}
			}

			// Perform the serialization
			iCalendarSerializer iserializer = new iCalendarSerializer();
			iserializer.iCalendar = icalendar;
			iserializer.Serialize(fileName);
		}


		private Dictionary<string, List<Item>> recurrentItems;
	}
}
